@@ -2,6 +2,7 @@ package filter
 
 import (
 	"path/filepath"
+	"strconv"
 	"sync"
 
 	"github.com/gobwas/glob"
@@ -21,64 +22,64 @@ type Expression interface {
 	RequiresParse() (Expression, bool)
 	// IsRestrictedToStacks returns true if the expression is restricted to stacks.
 	IsRestrictedToStacks() bool
+	// Negated returns the equivalent expression with negation flipped.
+	Negated() Expression
 }
 
-// PathFilter represents a path or glob filter (e.g., "./path/**/*" or "/absolute/path").
-type PathFilter struct {
+// Expressions is a slice of expressions.
+type Expressions []Expression
+
+// PathExpression represents a path or glob filter (e.g., "./path/**/*" or "/absolute/path").
+type PathExpression struct {
 	compiledGlob glob.Glob
 	compileErr   error
 	Value        string
-	WorkingDir   string
 	compileOnce  sync.Once
 }
 
 // NewPathFilter creates a new PathFilter with lazy glob compilation.
-func NewPathFilter(value string, workingDir string) *PathFilter {
-	return &PathFilter{Value: value, WorkingDir: workingDir}
+func NewPathFilter(value string) *PathExpression {
+	return &PathExpression{Value: value}
 }
 
 // CompileGlob returns the compiled glob pattern, compiling it on first call.
 // Subsequent calls return the cached compiled glob and any error.
 // Uses sync.Once for thread-safe lazy initialization.
-func (p *PathFilter) CompileGlob() (glob.Glob, error) {
+func (p *PathExpression) CompileGlob() (glob.Glob, error) {
 	p.compileOnce.Do(func() {
 		pattern := p.Value
-		if !filepath.IsAbs(pattern) {
-			pattern = filepath.Join(p.WorkingDir, pattern)
-		}
-
-		pattern = filepath.ToSlash(pattern)
+		pattern = filepath.Clean(filepath.ToSlash(pattern))
 		p.compiledGlob, p.compileErr = glob.Compile(pattern, '/')
 	})
 
 	return p.compiledGlob, p.compileErr
 }
 
-func (p *PathFilter) expressionNode()                       {}
-func (p *PathFilter) String() string                        { return p.Value }
-func (p *PathFilter) RequiresDiscovery() (Expression, bool) { return p, false }
-func (p *PathFilter) RequiresParse() (Expression, bool)     { return p, false }
-func (p *PathFilter) IsRestrictedToStacks() bool            { return false }
+func (p *PathExpression) expressionNode()                       {}
+func (p *PathExpression) String() string                        { return p.Value }
+func (p *PathExpression) RequiresDiscovery() (Expression, bool) { return p, false }
+func (p *PathExpression) RequiresParse() (Expression, bool)     { return p, false }
+func (p *PathExpression) IsRestrictedToStacks() bool            { return false }
+func (p *PathExpression) Negated() Expression                   { return NewPrefixExpression("!", p) }
 
-// AttributeFilter represents a key-value attribute filter (e.g., "name=my-app").
-type AttributeFilter struct {
+// AttributeExpression represents a key-value attribute filter (e.g., "name=my-app").
+type AttributeExpression struct {
 	compiledGlob glob.Glob
 	compileErr   error
 	Key          string
 	Value        string
-	WorkingDir   string
 	compileOnce  sync.Once
 }
 
-// NewAttributeFilter creates a new AttributeFilter with lazy glob compilation.
-func NewAttributeFilter(key string, value string, workingDir string) *AttributeFilter {
-	return &AttributeFilter{Key: key, Value: value, WorkingDir: workingDir}
+// NewAttributeExpression creates a new AttributeFilter with lazy glob compilation.
+func NewAttributeExpression(key string, value string) *AttributeExpression {
+	return &AttributeExpression{Key: key, Value: value}
 }
 
 // CompileGlob returns the compiled glob pattern for name and reading filters, compiling it on first call.
 // Returns an error if called on unsupported attributes (e.g. type, external).
 // Uses sync.Once for thread-safe lazy initialization.
-func (a *AttributeFilter) CompileGlob() (glob.Glob, error) {
+func (a *AttributeExpression) CompileGlob() (glob.Glob, error) {
 	// Only compile for attributes that support glob matching
 	if !a.supportsGlob() {
 		return nil, NewEvaluationError("attribute '" + a.Key + "' does not support glob patterns")
@@ -88,11 +89,7 @@ func (a *AttributeFilter) CompileGlob() (glob.Glob, error) {
 		pattern := a.Value
 
 		if a.Key == AttributeReading {
-			if !filepath.IsAbs(pattern) {
-				pattern = filepath.Join(a.WorkingDir, pattern)
-			}
-
-			pattern = filepath.ToSlash(pattern)
+			pattern = filepath.Clean(filepath.ToSlash(pattern))
 		}
 
 		a.compiledGlob, a.compileErr = glob.Compile(pattern, '/')
@@ -102,14 +99,14 @@ func (a *AttributeFilter) CompileGlob() (glob.Glob, error) {
 }
 
 // supportsGlob returns true if the attribute filter supports glob patterns.
-func (a *AttributeFilter) supportsGlob() bool {
+func (a *AttributeExpression) supportsGlob() bool {
 	return a.Key == AttributeReading || a.Key == AttributeName || a.Key == AttributeSource
 }
 
-func (a *AttributeFilter) expressionNode()                       {}
-func (a *AttributeFilter) String() string                        { return a.Key + "=" + a.Value }
-func (a *AttributeFilter) RequiresDiscovery() (Expression, bool) { return a, true }
-func (a *AttributeFilter) RequiresParse() (Expression, bool) {
+func (a *AttributeExpression) expressionNode()                       {}
+func (a *AttributeExpression) String() string                        { return a.Key + "=" + a.Value }
+func (a *AttributeExpression) RequiresDiscovery() (Expression, bool) { return a, true }
+func (a *AttributeExpression) RequiresParse() (Expression, bool) {
 	switch a.Key {
 	// All of these attributes can be determined based on the component + configuration filepath.
 	case AttributeName, AttributeType, AttributeExternal:
@@ -123,9 +120,12 @@ func (a *AttributeFilter) RequiresParse() (Expression, bool) {
 		return nil, true
 	}
 }
-func (a *AttributeFilter) IsRestrictedToStacks() bool {
+func (a *AttributeExpression) IsRestrictedToStacks() bool {
 	return a.Key == "type" && a.Value == "stack"
 }
+func (a *AttributeExpression) Negated() Expression {
+	return NewPrefixExpression("!", a)
+}
 
 // PrefixExpression represents a prefix operator expression (e.g., "!name=foo").
 type PrefixExpression struct {
@@ -150,7 +150,7 @@ func (p *PrefixExpression) IsRestrictedToStacks() bool {
 	switch p.Operator {
 	case "!":
 		switch a := p.Right.(type) {
-		case *AttributeFilter:
+		case *AttributeExpression:
 			switch a.Key {
 			case "type":
 				return a.Value != "stack"
@@ -164,6 +164,14 @@ func (p *PrefixExpression) IsRestrictedToStacks() bool {
 		return false
 	}
 }
+func (p *PrefixExpression) Negated() Expression {
+	switch p.Operator {
+	case "!":
+		return p.Right
+	default:
+		return NewPrefixExpression("!", p.Right)
+	}
+}
 
 // InfixExpression represents an infix operator expression (e.g., "./apps/* | name=bar").
 type InfixExpression struct {
@@ -211,13 +219,24 @@ func (i *InfixExpression) IsRestrictedToStacks() bool {
 		return false
 	}
 }
+func (i *InfixExpression) Negated() Expression {
+	switch i.Operator {
+	case "|":
+		return NewInfixExpression(i.Left.Negated(), i.Operator, i.Right)
+	default:
+		return NewInfixExpression(i.Left.Negated(), i.Operator, i.Right)
+	}
+}
 
-// GraphExpression represents a graph traversal expression (e.g., "...foo", "foo...", "...foo...", "^foo").
+// GraphExpression represents a graph traversal expression (e.g., "...foo", "foo...", "..1foo", "foo..2").
+// Depth fields control how many levels of dependencies/dependents to traverse.
 type GraphExpression struct {
 	Target              Expression
 	IncludeDependents   bool
 	IncludeDependencies bool
 	ExcludeTarget       bool
+	DependentDepth      int
+	DependencyDepth     int
 }
 
 // NewGraphExpression creates a new GraphExpression.
@@ -238,7 +257,12 @@ func NewGraphExpression(
 func (g *GraphExpression) expressionNode() {}
 func (g *GraphExpression) String() string {
 	result := ""
+
 	if g.IncludeDependents {
+		if g.DependentDepth > 0 {
+			result += strconv.Itoa(g.DependentDepth)
+		}
+
 		result += "..."
 	}
 
@@ -247,8 +271,13 @@ func (g *GraphExpression) String() string {
 	}
 
 	result += g.Target.String()
+
 	if g.IncludeDependencies {
 		result += "..."
+
+		if g.DependencyDepth > 0 {
+			result += strconv.Itoa(g.DependencyDepth)
+		}
 	}
 
 	return result
@@ -262,3 +291,56 @@ func (g *GraphExpression) RequiresParse() (Expression, bool) {
 	return g, true
 }
 func (g *GraphExpression) IsRestrictedToStacks() bool { return false }
+func (g *GraphExpression) Negated() Expression {
+	return NewPrefixExpression("!", g)
+}
+
+// GitExpression represents a Git-based filter expression (e.g., "[main...HEAD]" or "[main]").
+// It filters components based on changes between Git references.
+type GitExpression struct {
+	FromRef string
+	ToRef   string
+}
+
+func NewGitExpression(fromRef, toRef string) *GitExpression {
+	return &GitExpression{FromRef: fromRef, ToRef: toRef}
+}
+
+func (g *GitExpression) expressionNode() {}
+func (g *GitExpression) String() string {
+	return "[" + g.FromRef + "..." + g.ToRef + "]"
+}
+func (g *GitExpression) RequiresDiscovery() (Expression, bool) {
+	// Git filters require discovery to check which components changed between references
+	return g, true
+}
+func (g *GitExpression) RequiresParse() (Expression, bool) {
+	// Git filters don't require parsing - they compare file paths, not HCL content
+	return nil, false
+}
+func (g *GitExpression) IsRestrictedToStacks() bool { return false }
+func (g *GitExpression) Negated() Expression {
+	return NewPrefixExpression("!", g)
+}
+
+// GitExpressions is a slice of Git expressions.
+type GitExpressions []*GitExpression
+
+// UniqueGitRefs returns all unique Git references in a slice of expressions.
+func (e GitExpressions) UniqueGitRefs() []string {
+	refSet := make(map[string]struct{}, len(e))
+
+	for _, expr := range e {
+		refs := collectGitReferences(expr)
+		for _, ref := range refs {
+			refSet[ref] = struct{}{}
+		}
+	}
+
+	result := make([]string, 0, len(refSet))
+	for ref := range refSet {
+		result = append(result, ref)
+	}
+
+	return result
+}
