@@ -7,9 +7,9 @@ import (
 	"path"
 
 	"github.com/gruntwork-io/terragrunt/internal/remotestate/backend"
-	"github.com/gruntwork-io/terragrunt/options"
+	"github.com/gruntwork-io/terragrunt/internal/shell"
 	"github.com/gruntwork-io/terragrunt/pkg/log"
-	"github.com/gruntwork-io/terragrunt/shell"
+	"github.com/gruntwork-io/terragrunt/pkg/options"
 )
 
 const (
@@ -48,7 +48,7 @@ func (backend *Backend) NeedsBootstrap(ctx context.Context, l log.Logger, backen
 		bucketName = gcsCfg.Bucket
 	)
 
-	client, err := NewClient(ctx, extGCSCfg)
+	client, err := NewClient(ctx, l, extGCSCfg, opts)
 	if err != nil {
 		return false, err
 	}
@@ -74,7 +74,7 @@ func (backend *Backend) Bootstrap(ctx context.Context, l log.Logger, backendConf
 		return err
 	}
 
-	client, err := NewClient(ctx, extGCSCfg)
+	client, err := NewClient(ctx, l, extGCSCfg, opts)
 	if err != nil {
 		return err
 	}
@@ -129,7 +129,7 @@ func (backend *Backend) IsVersionControlEnabled(ctx context.Context, l log.Logge
 
 	var bucketName = extGCSCfg.RemoteStateConfigGCS.Bucket
 
-	client, err := NewClient(ctx, extGCSCfg)
+	client, err := NewClient(ctx, l, extGCSCfg, opts)
 	if err != nil {
 		return false, err
 	}
@@ -156,7 +156,7 @@ func (backend *Backend) Migrate(ctx context.Context, l log.Logger, srcBackendCon
 		dstBucketKey  = path.Join(dstExtGCSCfg.RemoteStateConfigGCS.Prefix, defaultTfState)
 	)
 
-	client, err := NewClient(ctx, srcExtGCSCfg)
+	client, err := NewClient(ctx, l, srcExtGCSCfg, opts)
 	if err != nil {
 		return err
 	}
@@ -176,7 +176,7 @@ func (backend *Backend) Delete(ctx context.Context, l log.Logger, backendConfig
 		prefix     = extGCSCfg.RemoteStateConfigGCS.Prefix
 	)
 
-	client, err := NewClient(ctx, extGCSCfg)
+	client, err := NewClient(ctx, l, extGCSCfg, opts)
 	if err != nil {
 		return err
 	}
@@ -198,7 +198,7 @@ func (backend *Backend) DeleteBucket(ctx context.Context, l log.Logger, backendC
 		return err
 	}
 
-	client, err := NewClient(ctx, extGCSCfg)
+	client, err := NewClient(ctx, l, extGCSCfg, opts)
 	if err != nil {
 		return err
 	}
