@@ -8,9 +8,12 @@ import (
 
 	"github.com/gruntwork-io/terragrunt/internal/component"
 	"github.com/gruntwork-io/terragrunt/internal/report"
-	"github.com/gruntwork-io/terragrunt/options"
-	"github.com/gruntwork-io/terragrunt/tf"
-	"github.com/gruntwork-io/terragrunt/util"
+	"github.com/gruntwork-io/terragrunt/internal/runner/run"
+	"github.com/gruntwork-io/terragrunt/internal/runner/run/creds"
+	"github.com/gruntwork-io/terragrunt/internal/runner/runcfg"
+	"github.com/gruntwork-io/terragrunt/internal/tf"
+	"github.com/gruntwork-io/terragrunt/internal/util"
+	"github.com/gruntwork-io/terragrunt/pkg/options"
 )
 
 // UnitStatus represents the status of a unit during execution.
@@ -42,7 +45,13 @@ func NewUnitRunnerFromComponent(unit *component.Unit) *UnitRunner {
 	return NewUnitRunner(unit)
 }
 
-func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.TerragruntOptions, r *report.Report) error {
+func (runner *UnitRunner) runTerragrunt(
+	ctx context.Context,
+	opts *options.TerragruntOptions,
+	r *report.Report,
+	cfg *runcfg.RunConfig,
+	credsGetter *creds.Getter,
+) error {
 	if runner.Unit.Execution == nil || runner.Unit.Execution.Logger == nil {
 		return nil
 	}
@@ -59,10 +68,17 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 		unitPath := runner.Unit.AbsolutePath()
 		unitPath = util.CleanPath(unitPath)
 
-		// Pass the discovery working directory for worktree scenarios
+		// Pass the discovery context fields for worktree scenarios
 		var ensureOpts []report.EndOption
-		if discoveryCtx := runner.Unit.DiscoveryContext(); discoveryCtx != nil && discoveryCtx.WorkingDir != "" {
-			ensureOpts = append(ensureOpts, report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir))
+
+		if discoveryCtx := runner.Unit.DiscoveryContext(); discoveryCtx != nil {
+			ensureOpts = append(
+				ensureOpts,
+				report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir),
+				report.WithRef(discoveryCtx.Ref),
+				report.WithCmd(discoveryCtx.Cmd),
+				report.WithArgs(discoveryCtx.Args),
+			)
 		}
 
 		if _, err := r.EnsureRun(runner.Unit.Execution.Logger, unitPath, ensureOpts...); err != nil {
@@ -73,16 +89,18 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 	// Use a unit-scoped detailed exit code so retries in this unit don't clobber global state
 	globalExitCode := tf.DetailedExitCodeFromContext(ctx)
 
-	var unitExitCode tf.DetailedExitCode
+	unitExitCode := tf.NewDetailedExitCodeMap()
 
-	ctx = tf.ContextWithDetailedExitCode(ctx, &unitExitCode)
+	ctx = tf.ContextWithDetailedExitCode(ctx, unitExitCode)
 
-	runErr := opts.RunTerragrunt(ctx, runner.Unit.Execution.Logger, opts, r)
+	runErr := run.Run(ctx, runner.Unit.Execution.Logger, opts, r, cfg, credsGetter)
 
-	// Only merge the final unit exit code when the unit run completed without error
-	// and the exit code isn't stuck at 1 from a prior retry attempt.
-	if runErr == nil && globalExitCode != nil && unitExitCode.Get() != tf.DetailedExitCodeError {
-		globalExitCode.Set(unitExitCode.Get())
+	// Store the unit exit code in the global map using the unit path as key
+	// Get the exit code from the unit-scoped map (stored with empty string key in run_cmd.go)
+	if globalExitCode != nil {
+		unitPath := filepath.Clean(runner.Unit.AbsolutePath())
+		code := unitExitCode.Get(opts.WorkingDir)
+		globalExitCode.Set(unitPath, code)
 	}
 
 	// End the run with appropriate result (only if report is not nil)
@@ -115,7 +133,13 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 }
 
 // Run executes a component.Unit right now.
-func (runner *UnitRunner) Run(ctx context.Context, opts *options.TerragruntOptions, r *report.Report) error {
+func (runner *UnitRunner) Run(
+	ctx context.Context,
+	opts *options.TerragruntOptions,
+	r *report.Report,
+	cfg *runcfg.RunConfig,
+	credsGetter *creds.Getter,
+) error {
 	runner.Status = Running
 
 	if runner.Unit.Execution == nil {
@@ -124,19 +148,25 @@ func (runner *UnitRunner) Run(ctx context.Context, opts *options.TerragruntOptio
 
 	if runner.Unit.Execution.AssumeAlreadyApplied {
 		if runner.Unit.Execution.Logger != nil {
-			runner.Unit.Execution.Logger.Debugf("Assuming unit %s has already been applied and skipping it", runner.Unit.Path())
+			runner.Unit.Execution.Logger.Debugf(
+				"Assuming unit %s has already been applied and skipping it",
+				runner.Unit.Path(),
+			)
 		}
 
 		return nil
 	}
 
-	if err := runner.runTerragrunt(ctx, runner.Unit.Execution.TerragruntOptions, r); err != nil {
+	if err := runner.runTerragrunt(ctx, runner.Unit.Execution.TerragruntOptions, r, cfg, credsGetter); err != nil {
 		return err
 	}
 
 	// convert terragrunt output to json
 	if runner.Unit.OutputJSONFile(runner.Unit.Execution.TerragruntOptions) != "" {
-		l, jsonOptions, err := runner.Unit.Execution.TerragruntOptions.CloneWithConfigPath(runner.Unit.Execution.Logger, runner.Unit.Execution.TerragruntOptions.TerragruntConfigPath)
+		l, jsonOptions, err := runner.Unit.Execution.TerragruntOptions.CloneWithConfigPath(
+			runner.Unit.Execution.Logger,
+			runner.Unit.Execution.TerragruntOptions.TerragruntConfigPath,
+		)
 		if err != nil {
 			return err
 		}
@@ -150,7 +180,7 @@ func (runner *UnitRunner) Run(ctx context.Context, opts *options.TerragruntOptio
 
 		// Use an ad-hoc report to avoid polluting the main report
 		adhocReport := report.NewReport()
-		if err := jsonOptions.RunTerragrunt(ctx, l, jsonOptions, adhocReport); err != nil {
+		if err := run.Run(ctx, l, jsonOptions, adhocReport, cfg, credsGetter); err != nil {
 			return err
 		}
 
