@@ -5,7 +5,6 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -13,23 +12,22 @@ import (
 	"slices"
 	"strings"
 
-	"github.com/gruntwork-io/terragrunt/internal/cli"
+	"github.com/gruntwork-io/terragrunt/internal/tf"
 
-	"github.com/gruntwork-io/terragrunt/tf"
+	"github.com/gruntwork-io/terragrunt/internal/util"
 
-	"github.com/gruntwork-io/terragrunt/util"
-
-	"github.com/gruntwork-io/terragrunt/config"
 	"github.com/gruntwork-io/terragrunt/internal/component"
 	tgerrors "github.com/gruntwork-io/terragrunt/internal/errors"
 	"github.com/gruntwork-io/terragrunt/internal/queue"
 	"github.com/gruntwork-io/terragrunt/internal/report"
 	"github.com/gruntwork-io/terragrunt/internal/runner/common"
-	"github.com/gruntwork-io/terragrunt/options"
+	"github.com/gruntwork-io/terragrunt/internal/runner/run/creds"
+	"github.com/gruntwork-io/terragrunt/internal/runner/run/creds/providers/externalcmd"
+	"github.com/gruntwork-io/terragrunt/internal/telemetry"
+	"github.com/gruntwork-io/terragrunt/pkg/config"
 	"github.com/gruntwork-io/terragrunt/pkg/log"
 	"github.com/gruntwork-io/terragrunt/pkg/log/format/placeholders"
-	"github.com/gruntwork-io/terragrunt/telemetry"
-	"github.com/hashicorp/hcl/v2"
+	"github.com/gruntwork-io/terragrunt/pkg/options"
 )
 
 // Runner implements the Stack interface for runner pool execution.
@@ -224,12 +222,8 @@ func resolveUnitsFromDiscovery(
 			}
 		}
 
-		// Initialize execution context
-		unit.Execution = &component.UnitExecution{
-			TerragruntOptions:    unitOpts,
-			Logger:               unitLogger,
-			FlagExcluded:         unit.Excluded(),
-			AssumeAlreadyApplied: false,
+		if unit.Execution == nil {
+			unit.Execution = component.NewUnitExecution(unitLogger, unitOpts, unit.Excluded())
 		}
 
 		// Store config from discovery context if available
@@ -262,7 +256,7 @@ func checkLocalStateWithGitRefs(l log.Logger, units []*component.Unit) {
 			continue
 		}
 
-		if unitConfig.RemoteState == nil || unitConfig.RemoteState.BackendName == "local" {
+		if unitConfig.RemoteState == nil || (unitConfig.RemoteState.Config != nil && unitConfig.RemoteState.BackendName == "local") {
 			l.Warnf(
 				"One or more units discovered using Git-based filter expressions (e.g. [HEAD~1...HEAD]) do not have a remote_state configuration. This may result in unexpected outcomes, such as outputs for dependencies returning empty. It is strongly recommended to use remote state when working with Git-based filter expressions.",
 			)
@@ -388,54 +382,6 @@ func filterUnitsToComponents(units []*component.Unit) component.Components {
 	return result
 }
 
-// Limit recursive descent when inspecting nested errors
-const maxConfigurationErrorDepth = 100
-
-// isConfigurationError checks if an error is a configuration/validation error
-// that should always cause command failure regardless of fail-fast setting.
-func isConfigurationError(err error) bool {
-	return isConfigurationErrorDepth(err, 0)
-}
-
-func isConfigurationErrorDepth(err error, depth int) bool {
-	if err == nil {
-		return false
-	}
-
-	if depth >= maxConfigurationErrorDepth {
-		return false
-	}
-
-	// Check for specific configuration error types
-	if tgerrors.IsError(err, config.ConflictingRunCmdCacheOptionsError{}) {
-		return true
-	}
-
-	// Inspect HCL diagnostics (structured errors) for run_cmd cache-option conflicts
-	for _, unwrapped := range tgerrors.UnwrapErrors(err) {
-		var diags hcl.Diagnostics
-		if errors.As(unwrapped, &diags) {
-			for _, d := range diags {
-				if d != nil && d.Severity == hcl.DiagError && d.Summary == "Error in function call" {
-					return true
-				}
-			}
-		}
-	}
-
-	// Check wrapped errors in MultiError
-	var multiErr *tgerrors.MultiError
-	if errors.As(err, &multiErr) {
-		for _, wrappedErr := range multiErr.WrappedErrors() {
-			if isConfigurationErrorDepth(wrappedErr, depth+1) {
-				return true
-			}
-		}
-	}
-
-	return false
-}
-
 // Run executes the stack according to TerragruntOptions and returns the first
 // error (or a joined error) once execution is finished.
 func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.TerragruntOptions) error {
@@ -480,10 +426,17 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// Ensure path is absolute for reporting
 				unitPath := u.AbsolutePath()
 
-				// Pass the discovery working directory for worktree scenarios
+				// Pass the discovery context fields for worktree scenarios
 				var ensureOpts []report.EndOption
-				if discoveryCtx := u.DiscoveryContext(); discoveryCtx != nil && discoveryCtx.WorkingDir != "" {
-					ensureOpts = append(ensureOpts, report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir))
+
+				if discoveryCtx := u.DiscoveryContext(); discoveryCtx != nil {
+					ensureOpts = append(
+						ensureOpts,
+						report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir),
+						report.WithRef(discoveryCtx.Ref),
+						report.WithCmd(discoveryCtx.Cmd),
+						report.WithArgs(discoveryCtx.Args),
+					)
 				}
 
 				run, err := r.Stack.Execution.Report.EnsureRun(l, unitPath, ensureOpts...)
@@ -532,7 +485,41 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 			u.Execution.TerragruntOptions.Writer = unitWriter
 			unitRunner := common.NewUnitRunner(u)
 
-			err := unitRunner.Run(childCtx, u.Execution.TerragruntOptions, r.Stack.Execution.Report)
+			// Use the unit's logger if populated, which has the proper context for logging.
+			unitLogger := u.Execution.Logger
+			if unitLogger == nil {
+				unitLogger = l
+			}
+
+			cfg, err := config.ReadTerragruntConfig(
+				childCtx,
+				unitLogger,
+				u.Execution.TerragruntOptions,
+				config.DefaultParserOptions(unitLogger, u.Execution.TerragruntOptions),
+			)
+			if err != nil {
+				return err
+			}
+
+			runCfg := cfg.ToRunConfig(unitLogger)
+
+			credsGetter := creds.NewGetter()
+			if err = credsGetter.ObtainAndUpdateEnvIfNecessary(
+				childCtx,
+				unitLogger,
+				u.Execution.TerragruntOptions,
+				externalcmd.NewProvider(unitLogger, u.Execution.TerragruntOptions),
+			); err != nil {
+				return err
+			}
+
+			err = unitRunner.Run(
+				childCtx,
+				u.Execution.TerragruntOptions,
+				r.Stack.Execution.Report,
+				runCfg,
+				credsGetter,
+			)
 
 			// Flush any remaining buffered output
 			if flushErr := unitWriter.Flush(); flushErr != nil && err == nil {
@@ -556,7 +543,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 
 	err := controller.Run(ctx, l)
 
-	// Emit report entries for early exit units after controller completes
+	// Emit report entries for early exit and failed units after controller completes
 	if r.Stack.Execution != nil && r.Stack.Execution.Report != nil {
 		// Build a quick lookup of queue entry status by path to avoid nested scans
 		statusByPath := make(map[string]queue.Status, len(r.queue.Entries))
@@ -565,25 +552,33 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 		}
 
 		for _, entry := range r.queue.Entries {
-			if entry.Status == queue.StatusEarlyExit {
+			// Handle both early exit and failed units to ensure they're in the report
+			if entry.Status == queue.StatusEarlyExit || entry.Status == queue.StatusFailed {
 				unit := r.Stack.FindUnitByPath(entry.Component.Path())
 				if unit == nil {
-					l.Warnf("Could not find unit for early exit entry: %s", entry.Component.Path())
+					l.Warnf("Could not find unit for entry: %s", entry.Component.Path())
 					continue
 				}
 
 				// Ensure path is absolute for reporting
 				unitPath := unit.AbsolutePath()
 
-				// Pass the discovery working directory for worktree scenarios
+				// Pass the discovery context fields for worktree scenarios
 				var ensureOpts []report.EndOption
-				if discoveryCtx := unit.DiscoveryContext(); discoveryCtx != nil && discoveryCtx.WorkingDir != "" {
-					ensureOpts = append(ensureOpts, report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir))
+
+				if discoveryCtx := unit.DiscoveryContext(); discoveryCtx != nil {
+					ensureOpts = append(
+						ensureOpts,
+						report.WithDiscoveryWorkingDir(discoveryCtx.WorkingDir),
+						report.WithRef(discoveryCtx.Ref),
+						report.WithCmd(discoveryCtx.Cmd),
+						report.WithArgs(discoveryCtx.Args),
+					)
 				}
 
 				run, reportErr := r.Stack.Execution.Report.EnsureRun(l, unitPath, ensureOpts...)
 				if reportErr != nil {
-					l.Errorf("Error ensuring run for early exit unit %s: %v", unitPath, reportErr)
+					l.Errorf("Error ensuring run for unit %s: %v", unitPath, reportErr)
 					continue
 				}
 
@@ -604,43 +599,43 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 					}
 				}
 
-				endOpts := []report.EndOption{
-					report.WithResult(report.ResultEarlyExit),
-					report.WithReason(report.ReasonAncestorError),
-				}
-				if failedAncestor != "" {
-					endOpts = append(endOpts, report.WithCauseAncestorExit(failedAncestor))
-				}
+				switch entry.Status { //nolint:exhaustive
+				case queue.StatusEarlyExit:
+					endOpts := []report.EndOption{
+						report.WithResult(report.ResultEarlyExit),
+						report.WithReason(report.ReasonAncestorError),
+					}
+					if failedAncestor != "" {
+						endOpts = append(endOpts, report.WithCauseAncestorExit(failedAncestor))
+					}
 
-				if endErr := r.Stack.Execution.Report.EndRun(l, run.Path, endOpts...); endErr != nil {
-					l.Errorf("Error ending run for early exit unit %s: %v", unitPath, endErr)
+					if endErr := r.Stack.Execution.Report.EndRun(l, run.Path, endOpts...); endErr != nil {
+						l.Errorf("Error ending run for early exit unit %s: %v", unitPath, endErr)
+					}
+				case queue.StatusFailed:
+					// For failed units, check if they failed due to dependency errors
+					// If so, mark them as early exit; otherwise mark as failed
+					endOpts := []report.EndOption{
+						report.WithResult(report.ResultFailed),
+						report.WithReason(report.ReasonRunError),
+					}
+					if failedAncestor != "" {
+						// If a dependency failed, treat this as early exit due to ancestor error
+						endOpts = []report.EndOption{
+							report.WithResult(report.ResultEarlyExit),
+							report.WithReason(report.ReasonAncestorError),
+							report.WithCauseAncestorExit(failedAncestor),
+						}
+					}
+
+					if endErr := r.Stack.Execution.Report.EndRun(l, run.Path, endOpts...); endErr != nil {
+						l.Errorf("Error ending run for failed unit %s: %v", unitPath, endErr)
+					}
 				}
 			}
 		}
 	}
 
-	// Handle errors based on fail-fast mode and error type
-	// Configuration errors always fail regardless of --fail-fast
-	// Execution errors are suppressed when --fail-fast is not set
-	if err != nil {
-		if isConfigurationError(err) || opts.FailFast {
-			// Configuration errors or fail-fast mode: propagate error
-			return err
-		}
-
-		// Execution errors without fail-fast: log but don't fail
-		l.Errorf("Run failed: %v", err)
-
-		// Set detailed exit code if context has one
-		exitCode := tf.DetailedExitCodeFromContext(ctx)
-		if exitCode != nil {
-			exitCode.Set(int(cli.ExitCodeGeneralError))
-		}
-
-		// Return nil to indicate success (no --fail-fast) but errors were logged
-		return nil
-	}
-
 	return err
 }
 
@@ -679,15 +674,19 @@ func (r *Runner) LogUnitDeployOrder(l log.Logger, terraformCommand string) error
 	showAbsPaths := r.Stack.Execution != nil && r.Stack.Execution.TerragruntOptions != nil &&
 		r.Stack.Execution.TerragruntOptions.LogShowAbsPaths
 
+	var outStrSb strings.Builder
+
 	for _, unit := range entries {
 		unitPath := unit.Component.DisplayPath()
 		if showAbsPaths {
 			unitPath = unit.Component.Path()
 		}
 
-		outStr += fmt.Sprintf("- Unit %s\n", unitPath)
+		fmt.Fprintf(&outStrSb, "- Unit %s\n", unitPath)
 	}
 
+	outStr += outStrSb.String()
+
 	l.Info(outStr)
 
 	return nil
@@ -724,7 +723,7 @@ func (r *Runner) ListStackDependentUnits() map[string][]string {
 	for _, unit := range r.queue.Entries {
 		if len(unit.Component.Dependencies()) != 0 {
 			for _, dep := range unit.Component.Dependencies() {
-				dependentUnits[dep.Path()] = util.RemoveDuplicatesFromList(append(dependentUnits[dep.Path()], unit.Component.Path()))
+				dependentUnits[dep.Path()] = util.RemoveDuplicates(append(dependentUnits[dep.Path()], unit.Component.Path()))
 			}
 		}
 	}
@@ -735,8 +734,8 @@ func (r *Runner) ListStackDependentUnits() map[string][]string {
 		for unit, dependents := range dependentUnits {
 			for _, dependent := range dependents {
 				initialSize := len(dependentUnits[unit])
-				list := util.RemoveDuplicatesFromList(append(dependentUnits[unit], dependentUnits[dependent]...))
-				list = util.RemoveElementFromList(list, unit)
+				list := util.RemoveDuplicates(append(dependentUnits[unit], dependentUnits[dependent]...))
+				list = slices.DeleteFunc(list, func(path string) bool { return path == unit })
 				dependentUnits[unit] = list
 
 				if initialSize != len(dependentUnits[unit]) {
@@ -763,7 +762,6 @@ func (r *Runner) syncTerraformCliArgs(l log.Logger, opts *options.TerragruntOpti
 		discoveryCtx := unit.DiscoveryContext()
 		if discoveryCtx != nil && len(discoveryCtx.Args) > 0 {
 			mergedArgs := slices.Clone(unit.Execution.TerragruntOptions.TerraformCliArgs)
-
 			for _, stackArg := range opts.TerraformCliArgs {
 				if stackArg == opts.TerraformCliArgs.First() {
 					continue
@@ -780,9 +778,16 @@ func (r *Runner) syncTerraformCliArgs(l log.Logger, opts *options.TerragruntOpti
 		}
 
 		planFile := unit.PlanFile(opts)
+
 		if planFile != "" {
 			l.Debugf("Using output file %s for unit %s", planFile, unit.Execution.TerragruntOptions.TerragruntConfigPath)
 
+			if slices.ContainsFunc(unit.Execution.TerragruntOptions.TerraformCliArgs, isPlanArg) {
+				// Plan file already present, just reorder to ensure it's at the end
+				unit.Execution.TerragruntOptions.TerraformCliArgs = reorderPlanFilesToEnd(unit.Execution.TerragruntOptions.TerraformCliArgs)
+				continue
+			}
+
 			if unit.Execution.TerragruntOptions.TerraformCommand == tf.CommandNamePlan {
 				// for plan command add -out=<file> to the terraform cli args
 				unit.Execution.TerragruntOptions.TerraformCliArgs = append(unit.Execution.TerragruntOptions.TerraformCliArgs, "-out="+planFile)
@@ -791,9 +796,56 @@ func (r *Runner) syncTerraformCliArgs(l log.Logger, opts *options.TerragruntOpti
 
 			unit.Execution.TerragruntOptions.TerraformCliArgs = append(unit.Execution.TerragruntOptions.TerraformCliArgs, planFile)
 		}
+
+		// Always reorder to ensure plan files are at the end, after all flags
+		unit.Execution.TerragruntOptions.TerraformCliArgs = reorderPlanFilesToEnd(unit.Execution.TerragruntOptions.TerraformCliArgs)
 	}
 }
 
+func isPlanArg(arg string) bool {
+	if strings.HasPrefix(arg, "-") {
+		return false
+	}
+
+	if strings.HasSuffix(arg, ".tfplan") || strings.HasSuffix(arg, ".plan") {
+		return true
+	}
+
+	return false
+}
+
+// isPreviousArgFlag checks if the previous argument is a flag that might expect a value.
+// A flag without '=' (like "-out") expects its value as the next argument.
+func isPreviousArgFlag(args []string, index int) bool {
+	if index <= 0 {
+		return false
+	}
+
+	prevArg := args[index-1]
+	// If previous arg is a flag without '=', current arg might be its value
+	return strings.HasPrefix(prevArg, "-") && !strings.Contains(prevArg, "=")
+}
+
+// reorderPlanFilesToEnd moves plan file arguments to the end of the args slice,
+// preserving flag-value pairs (e.g., "-out plan.plan" stays together).
+func reorderPlanFilesToEnd(args []string) []string {
+	regularArgs := make([]string, 0, len(args))
+
+	var planArgs []string
+
+	for i, arg := range args {
+		// Move to end if: looks like plan file AND previous arg isn't a flag expecting a value
+		if isPlanArg(arg) && !isPreviousArgFlag(args, i) {
+			planArgs = append(planArgs, arg)
+			continue
+		}
+
+		regularArgs = append(regularArgs, arg)
+	}
+
+	return append(regularArgs, planArgs...)
+}
+
 // summarizePlanAllErrors summarizes all errors encountered during the plan phase across all units in the stack.
 func (r *Runner) summarizePlanAllErrors(l log.Logger, errorStreams []bytes.Buffer) {
 	for i, errorStream := range errorStreams {
@@ -810,7 +862,7 @@ func (r *Runner) summarizePlanAllErrors(l log.Logger, errorStreams []bytes.Buffe
 			unit := r.Stack.Units[i]
 			if len(unit.Dependencies()) > 0 {
 				cfg := unit.Config()
-				if cfg != nil && cfg.Dependencies != nil {
+				if cfg != nil && cfg.Dependencies != nil && len(cfg.Dependencies.Paths) > 0 {
 					dependenciesMsg = fmt.Sprintf(" contains dependencies to %v and", cfg.Dependencies.Paths)
 				} else {
 					dependenciesMsg = " contains dependencies and"
